@@ -7,7 +7,7 @@
 
 #written by Jeremy M. Beaulieu and Jeffrey C. Oliver
 
-ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), rate.cat, ntraits=NULL, rate.mat=NULL, model="ARD", root.p=NULL, get.likelihood=FALSE, get.tip.states = FALSE){
+ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), rate.cat, ntraits=NULL, rate.mat=NULL, model="ARD", root.p=NULL, get.likelihood=FALSE, get.tip.states = FALSE, collapse = TRUE){
     
   # if(hasArg(corHMM_fit)){
   #   corHMM_fit$phy$node.label <- NULL
@@ -36,7 +36,7 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), rate
     
     #data consistency stuff
     input.data <- data
-    corData <- corProcessData(data)
+    corData <- corProcessData(data, collapse = collapse)
     data <- corData$corData
 
     matching <- match.tree.data(phy,data)
@@ -57,11 +57,11 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), rate
     k <- ntraits <- length(corData$ObservedTraits)
     drop.states = NULL
     if(is.null(rate.mat)){
-        model.set.final <- rate.cat.set.corHMM.JDB(phy=phy,data=input.data, rate.cat=rate.cat, ntraits = ntraits, model = model)
+        model.set.final <- rate.cat.set.corHMM.JDB(phy=phy,data=input.data, rate.cat=rate.cat, ntraits = ntraits, model = model, rate.mat = rate.mat, collapse = collapse)
         rate.mat <- model.set.final$index.matrix
         rate <- model.set.final$rate
     }else{
-        model.set.final <- rate.cat.set.corHMM.JDB(phy=phy,data=input.data, rate.cat=rate.cat, ntraits = ntraits, model = model)
+        model.set.final <- rate.cat.set.corHMM.JDB(phy=phy,data=input.data, rate.cat=rate.cat, ntraits = ntraits, model = model, rate.mat=rate.mat, collapse = collapse)
         rate <- rate.mat
         col.sums <- which(colSums(rate.mat, na.rm=TRUE) == 0)
         row.sums <- which(rowSums(rate.mat, na.rm=TRUE) == 0)
@@ -95,10 +95,10 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), rate
     TIPS <- 1:nb.tip
     anc <- unique(phy$edge[,1])
     
-    # if the q matrix has columns not estimated, remove them
-    row2rm <- apply(rate, 1, function(x) all(x == max(rate)))
-    col2rm <- apply(rate, 2, function(x) all(x == max(rate)))
-    Q.root <- Q[!row2rm | !col2rm, !row2rm | !col2rm]
+    # # if the q matrix has columns not estimated, remove them
+    # row2rm <- apply(rate, 1, function(x) all(x == max(rate)))
+    # col2rm <- apply(rate, 2, function(x) all(x == max(rate)))
+    # Q.root <- Q[!row2rm | !col2rm, !row2rm | !col2rm]
     
     if(method=="joint"){
         if(!is.null(phy$node.label)){
@@ -152,11 +152,11 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), rate
                 }
                 if(is.na(known.state.vector[focal])){
                     equil.root <- NULL
-                    for(i in 1:ncol(Q.root)){
-                        posrows <- which(Q.root[,i] >= 0)
-                        rowsum <- sum(Q.root[posrows,i])
-                        poscols <- which(Q.root[i,] >= 0)
-                        colsum <- sum(Q.root[i,poscols])
+                    for(i in 1:ncol(Q)){
+                        posrows <- which(Q[,i] >= 0)
+                        rowsum <- sum(Q[posrows,i])
+                        poscols <- which(Q[i,] >= 0)
+                        colsum <- sum(Q[i,poscols])
                         equil.root <- c(equil.root,rowsum/(rowsum+colsum))
                     }
                     if (is.null(root.p)){
@@ -169,7 +169,7 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), rate
                             root.p <- flat.root
                             pupko.L[focal, ] <- root.state
                         }else{
-                            root.p <- rep(0, dim(Q.root)[2])
+                            root.p <- rep(0, dim(Q)[2])
                             root.p[known.state.vector[focal]] <- 1
                             pupko.L[focal, ] <- root.state
                         }
@@ -178,12 +178,12 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), rate
                         if(is.character(root.p)){
                             # root.p==yang will fix root probabilities based on the inferred rates: q10/(q01+q10), q01/(q01+q10), etc.
                             if(root.p == "yang"){
-                                root.p <- Null(Q.root)
+                                root.p <- Null(Q)
                                 root.p <- c(root.p/sum(root.p))
                                 pupko.L[focal, ] <- root.state
                             }else{
                                 # root.p==maddfitz will fix root probabilities according to FitzJohn et al 2009 Eq. 10:
-                                root.p <- (root.state / sum(root.state))[!col2rm]
+                                root.p <- (root.state / sum(root.state))[ ]
                                 pupko.L[focal,] <- root.state
                             }
                         }
@@ -194,7 +194,7 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), rate
                         }
                     }
                 }else{
-                    root.p = rep(0, dim(Q.root)[1])
+                    root.p = rep(0, dim(Q)[1])
                     root.p[known.state.vector[focal]] <- 1
                     pupko.L[focal, ] <- root.state
                 }
@@ -252,12 +252,12 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), rate
         }
         root <- nb.tip + 1L
         if(get.likelihood == TRUE){
-            loglik <- log(sum(exp(log(root.p)+log(pupko.L[root,!col2rm]))))
+            loglik <- log(sum(exp(log(root.p)+log(pupko.L[root, ]))))
             return(as.numeric(loglik))
         }else{
             root <- nb.tip + 1L
             if(is.na(known.state.vector[root])){
-                pupko.L[root,!col2rm] <- log(root.p)+log(pupko.L[root,!col2rm])
+                pupko.L[root, ] <- log(root.p)+log(pupko.L[root, ])
                 lik.states[root] <- which(pupko.L[root,] == max(pupko.L[root,]))[1]
             }else{
                 lik.states[root] <- known.state.vector[root]
@@ -315,11 +315,11 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), rate
         root <- nb.tip + 1L
         #Enter the root defined root probabilities if they are supplied by the user:
         equil.root <- NULL
-        for(i in 1:ncol(Q.root)){
-            posrows <- which(Q.root[,i] >= 0)
-            rowsum <- sum(Q.root[posrows,i])
-            poscols <- which(Q.root[i,] >= 0)
-            colsum <- sum(Q.root[i,poscols])
+        for(i in 1:ncol(Q)){
+            posrows <- which(Q[,i] >= 0)
+            rowsum <- sum(Q[posrows,i])
+            poscols <- which(Q[i,] >= 0)
+            colsum <- sum(Q[i,poscols])
             equil.root <- c(equil.root,rowsum/(rowsum+colsum))
         }
         if (is.null(root.p)){
@@ -327,32 +327,32 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), rate
             k.rates <- 1/length(which(!is.na(equil.root)))
             flat.root[!is.na(flat.root)] = k.rates
             flat.root[is.na(flat.root)] = 0
-            liks.down[root, !col2rm] <- flat.root * liks.down[root, !col2rm]
-            liks.down[root, !col2rm] <- liks.down[root,!col2rm] / sum(liks.down[root, !col2rm])
+            liks.down[root,  ] <- flat.root * liks.down[root,  ]
+            liks.down[root,  ] <- liks.down[root, ] / sum(liks.down[root,  ])
             root.p = flat.root
         }else{
             if(is.character(root.p)){
                 # root.p==yang will fix root probabilities based on the inferred rates: q10/(q01+q10), q01/(q01+q10), etc.
                 if(root.p == "yang"){
-                    root.p <- Null(Q.root)
+                    root.p <- Null(Q)
                     root.p <- c(root.p/sum(root.p))
-                    liks.down[root, !col2rm] <-  liks.down[root, !col2rm] * root.p
-                    liks.down[root, !col2rm] <- liks.down[root,!col2rm] / sum(liks.down[root,!col2rm])
+                    liks.down[root,  ] <-  liks.down[root,  ] * root.p
+                    liks.down[root,  ] <- liks.down[root, ] / sum(liks.down[root, ])
                 }else{
                     # root.p==maddfitz will fix root probabilities according to FitzJohn et al 2009 Eq. 10:
-                    root.p = liks.down[root,!col2rm] / sum(liks.down[root,!col2rm])
-                    liks.down[root, !col2rm] <- root.p * liks.down[root, !col2rm]
-                    liks.down[root, !col2rm] <- liks.down[root,!col2rm] / sum(liks.down[root, !col2rm])
+                    root.p = liks.down[root, ] / sum(liks.down[root, ])
+                    liks.down[root,  ] <- root.p * liks.down[root,  ]
+                    liks.down[root,  ] <- liks.down[root, ] / sum(liks.down[root,  ])
                 }
             }else{
-                liks.down[root, !col2rm] <- root.p * liks.down[root, !col2rm]
-                liks.down[root, !col2rm] <- liks.down[root,!col2rm] / sum(liks.down[root, !col2rm])
+                liks.down[root,  ] <- root.p * liks.down[root,  ]
+                liks.down[root,  ] <- liks.down[root, ] / sum(liks.down[root,  ])
             }
         }
         #The up-pass
         obsRoot <- root.p
         root.p <- vector("numeric", dim(liks)[2])
-        root.p[!col2rm] <- obsRoot
+        root.p[ ] <- obsRoot
         liks.up <- liks
         states<-apply(liks,1,which.max)
         N <- dim(phy$edge)[1]
@@ -411,7 +411,7 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), rate
         }
         if(get.tip.states == TRUE){
             #Now get the states for the tips (will do, not available for general use):
-            liks.final[TIPS,] <- GetTipStateBruteForce(p=p, phy=phy, data=input.data, rate.mat=rate.mat, rate.cat=rate.cat, ntraits=ntraits, model=model, root.p=root.p_input)
+            liks.final[TIPS,] <- GetTipStateBruteForce(p=p, phy=phy, data=input.data, rate.mat=rate.mat, rate.cat=rate.cat, ntraits=ntraits, model=model, root.p=root.p_input, collapse = collapse)
         }else{
             liks.final[TIPS,] <- liks.down[TIPS,]
         }
@@ -454,11 +454,11 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), rate
             liks[focal, ] <- v/comp[focal]
         }
         equil.root <- NULL
-        for(i in 1:ncol(Q.root)){
-            posrows <- which(Q.root[,i] >= 0)
-            rowsum <- sum(Q.root[posrows,i])
-            poscols <- which(Q.root[i,] >= 0)
-            colsum <- sum(Q.root[i,poscols])
+        for(i in 1:ncol(Q)){
+            posrows <- which(Q[,i] >= 0)
+            rowsum <- sum(Q[posrows,i])
+            poscols <- which(Q[i,] >= 0)
+            colsum <- sum(Q[i,poscols])
             equil.root <- c(equil.root,rowsum/(rowsum+colsum))
         }
         if (is.null(root.p)){
@@ -466,27 +466,27 @@ ancRECON <- function(phy, data, p, method=c("joint", "marginal", "scaled"), rate
             k.rates <- 1/length(which(!is.na(equil.root)))
             flat.root[!is.na(flat.root)] = k.rates
             flat.root[is.na(flat.root)] = 0
-            liks[root,!col2rm] <- flat.root * liks[root,!col2rm]
-            liks[root,!col2rm] <- liks[root,!col2rm] / sum(liks[root,!col2rm])
+            liks[root, ] <- flat.root * liks[root, ]
+            liks[root, ] <- liks[root, ] / sum(liks[root, ])
         }else{
             if(is.character(root.p)){
                 # root.p==yang will fix root probabilities based on the inferred rates: q10/(q01+q10), q01/(q01+q10), etc.
                 if(root.p == "yang"){
-                    root.p <- Null(Q.root)
+                    root.p <- Null(Q)
                     root.p <- c(root.p/sum(root.p))
-                    liks[root,!col2rm] <- root.p * liks[root, !col2rm]
-                    liks[root,!col2rm] <- liks[root,!col2rm] / sum(liks[root,!col2rm])
+                    liks[root, ] <- root.p * liks[root,  ]
+                    liks[root, ] <- liks[root, ] / sum(liks[root, ])
                 }else{
                     # root.p==maddfitz will fix root probabilities according to FitzJohn et al 2009 Eq. 10:
-                    root.p = liks[root,!col2rm] / sum(liks[root,!col2rm])
-                    liks[root,!col2rm] <- root.p * liks[root,!col2rm]
-                    liks[root,!col2rm] <- liks[root,!col2rm] / sum(liks[root,!col2rm])
+                    root.p = liks[root, ] / sum(liks[root, ])
+                    liks[root, ] <- root.p * liks[root, ]
+                    liks[root, ] <- liks[root, ] / sum(liks[root, ])
                 }
             }
             # root.p!==NULL will fix root probabilities based on user supplied vector:
             else{
-                liks[root,!col2rm] <- root.p * liks[root,!col2rm]
-                liks[root,!col2rm] <- liks[root,!col2rm] / sum(liks[root,!col2rm])
+                liks[root, ] <- root.p * liks[root, ]
+                liks[root, ] <- liks[root, ] / sum(liks[root, ])
             }
         }
         #Reports the probabilities for all internal nodes as well as tips:
@@ -516,12 +516,12 @@ getInfoPerNode <- function(lik.anc.states, Q){
   return(Info)
 }
 
-GetTipStateBruteForce <- function(p, phy, data, rate.mat, rate.cat, ntraits, model, root.p){
+GetTipStateBruteForce <- function(p, phy, data, rate.mat, rate.cat, ntraits, model, root.p, collapse = TRUE){
     
     nb.tip <- length(phy$tip.label)
     nb.node <- phy$Nnode
     
-    data.for.likelihood.function <- rate.cat.set.corHMM.JDB(phy=phy, data=data, rate.cat=rate.cat, ntraits = ntraits, model = model)
+    data.for.likelihood.function <- rate.cat.set.corHMM.JDB(phy=phy, data=data, rate.cat=rate.cat, ntraits = ntraits, model = model, collapse = collapse)
     
     if(!is.null(rate.mat)){
         rate <- rate.mat
